<?php

/**
 * Model for SysNagios GUI extension
 *
 **/
class SysNagios_Model
{

	private static $lock_file_path = "/tmp/getconfig.lock";

	/**
	 * Parses host-address
	 *
	 * @param $address string
	 * @return array
	 * @throws Exception
	 **/
	public function parse_host_address($address)
	{
		$regex = array(
			"ip" => '/^([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])\.([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])\.([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])\.([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])$/',
			"fqdn" => '/^(.*?)\.(.+)/',
			"hostname" => '/^([^.]*)$/'
		);
		$address_type = null;
		foreach ($regex as $idx => $reg) {
			preg_match($reg, $address, $match);
			// Find address type
			if (isset($match[0])) {
				$address_type = $idx;
				$matches = $match;
				break;
			}
		}
		$host_address = array();
		if (!isset($address_type)) {
			throw new Exception("Unknown address format");
		}
		switch ($address_type) {
			case 'ip':
				$host_address['ip'] = $matches[0];
				$host_address['fqdn'] = gethostbyaddr($host_address['ip']);
				if (preg_match($regex['fqdn'], $host_address['fqdn'], $matches) !== 0) {
					$host_address['hostname'] = $matches[1];
				} else {
					$host_address['hostname'] = $host_address['ip'];
				}
				break;
			case 'fqdn':
				$host_address['ip'] = gethostbyname($matches[0]);
				$host_address['fqdn'] = $matches[0];
				$host_address['hostname'] = $matches[1];
				break;
			case 'hostname':
				$host_address['ip'] = gethostbyname($matches[0]);
				$host_address['hostname'] = $matches[0];
				break;
			default:
				throw new Exception("Unknown address type.");
				break;
		}
		if (!isset($host_address['ip'])) {
			throw new Exception("No usable address was returned");
		}
		return $host_address;
	}

	/**
	 * Given a hostname, returns hostgroup membership
	 *
	 * @param $host string
	 * @return array
	 **/
	public function get_hostgroup_from_member($host)
	{
		if (!$host) {
			return false;
		}
		$hostgroups = array();
		$hgs = HostGroupPool_Model::all();
		$hgs = $hgs->reduce_by('members', $host, '>=');
		foreach ($hgs->it(array("name"), array()) as $hostgroup) {
			$hostgroups[] = $hostgroup->get_name();
		}
		return $hostgroups;
	}

	/**
	 * Creates a lock-file on a node
	 *
	 * @param $node string
	 * @return boolean
	 **/
	public function create_lock_file($node)
	{
		$cmd = "touch " . self::$lock_file_path;
		try {
			self::mon_node_ctrl($node, $cmd);
		} catch(Exception $e) {
			return $e->getMessage();
		}
		return true;
	}

	/**
	 * Removes lock-file on a node
	 *
	 * @param $node string
	 * @return boolean
	 **/
	public function remove_lock_file($node)
	{
		$cmd = "rm -f " . self::$lock_file_path;
		try {
			self::mon_node_ctrl($node, $cmd);
		} catch(Exception $e) {
			return $e->getMessage();
		}
		return true;
	}

	/**
	 * Check for existence of lockfile
	 *
	 * @param $node string
	 * @return boolean
	 **/
	public function lock_file_exists($node)
	{
		if ($node === "--self") {
			exec("test -f " . self::$lock_file_path, $output, $status);
		} else {
			exec("asmonitor mon node ctrl $node -- 'test -f " . self::$lock_file_path . "'", $output, $status);
		}
		if ($status === 0) {
			return true;
		}
		return false;
	}

	/**
	 * Checks for getconfigs.sh existance
	 *
	 * @param $path string
	 * @return boolean
	 **/
	public function getconfigs_exist($path, $node)
	{
		if ($node === "--self") {
			exec("test -f " . $path, $output, $status);
		} else {
			exec("asmonitor mon node ctrl $node -- 'test -f " . $path . "'", $output, $status);
		}
		if ($status === 0) {
			return true;
		}
		return false;
	}

	/**
	 * Fetches and parses nodes returned by mon node show
	 *
	 * @return array
	 **/
	function get_node_list()
	{
		$mon_node_show = self::mon_node_show();
		$nodes = self::parse_mon_node_show($mon_node_show);
		return $nodes;
	}

	/**
	 * Matches an existing host with responisble poller
	 * If no node is found with a corresponding hostgroup
	 * return --self as node
	 *
	 * @param $host string
	 * @param $hostgroups array
	 * @param $nodes array
	 * @return string
	 **/
	public function match_node($host, $hostgroups, $nodes)
	{
		foreach ($nodes as $node) {
			foreach ($hostgroups as $hostgroup) {
				if (isset($node['hostgroup']) && in_array($hostgroup, $node['hostgroup'])) {
					return $node['name'];
				}
			}
		}
		return "--self";
	}


	/***********************************************************************
	 * Helpers
	 **********************************************************************/

	/**
	 * Executes and returns output from mon node show
	 *
	 * @return array
	 * @throws Exception
	 **/
	public function mon_node_show()
	{
		exec("mon node show", $output, $status);
		if ($status !== 0) {
			throw new Exception("mon node show failed with status: $status and output: " . var_export($output, true));
		}
		return $output;
	}

	/**
	 * Parses merlin.conf and returns peers/pollers with corresponding hostgroups.
	 *
	 * @param $mon_node_show array
	 * @return array
	 * @throws Exception
	 **/
	public function parse_mon_node_show($mon_node_show)
	{
		$nodes = array();
		$node = "";
		foreach ($mon_node_show as $line) {
			if (substr($line, 0, 1) === "#") {
				$node = substr($line, 2);
				continue;
			} else if ($line !== "") {
				if ($line === "No nodes configured") {
					throw new Exception("No nodes are configured on this server.");
				}
				list($key, $value) = explode("=", $line);
				if ($key === "HOSTGROUP") {
					$value = explode(",", $value);
				}
				$nodes[$node][strtolower($key)] = $value;
			}
		}
		return $nodes;
	}

	/**
	 * mon node ctrl needn't be used on self. Just execute command
	 *
	 * @param $cmd string
	 * @return array
	 * @throws Exception
	 **/
	public function mon_node_ctrl_self($cmd)
	{
		exec($cmd, $output, $status);
		if ($status !== 0) {
			throw new Exception("mon node ctrl to node --self failed with exit code: " . $status . " and output: \n" . var_export($output, true));
		}
		return $output;
	}

	/**
	 * Send command to node utilizing mon node ctrl and return output
	 *
	 * @param $node string
	 * @param $cmd string
	 * @return array
	 * @throws Exception
	 **/
	public function mon_node_ctrl($node, $cmd, $exclusive = false)
	{
		if ($node === "--self") {
			return self::mon_node_ctrl_self($cmd);
		}
		exec("asmonitor mon node ctrl $node -- '$cmd'", $output, $status);
		if ($status !== 0) {
			throw new Exception("mon node ctrl to node " . $node . " failed with exit code: " . $status . " and output: \n" . var_export($output, true));
		}
		return $output;
	}

} // END class
